/*
 * This file is a part of Winix
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/* 
 * Copyright (c) 2010-2014, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
 
#include "item.h"
#include "misc.h"
#include "crypt.h"


namespace Winix
{




Item::Item()
{
	Clear();
}


// !! IMPROVE ME
// now we have Request::start_time and Request::start_date
// we can somehow get the current time from the request
// may setting the date should be completetly removed from here?
void Item::SetDateToNow()
{
	date_creation     = std::time(0);
	date_modification = date_creation;
}


void Item::SetDateModifyToNow()
{
	date_modification = std::time(0);
}


void Item::Clear()
{
	id = -1;
	
	user_id = -1;
	group_id = -1;
	privileges = 0;
	modification_user_id = -1;

	guest_name.clear();
	
	subject.clear();
	content.clear();
	modify_index = 0;
	url.clear();	
	
	content_type = ct_formatted_text;
	
	type = none;
	parent_id = -1;

	link_to.clear();
	link_redirect = false;

	content_id = -1;
	ref = 1;

	file_path.clear();
	file_fs   = -1;
	file_type = WINIX_ITEM_FILETYPE_NONE;
	hash.clear();
	hash_type = WINIX_CRYPT_HASH_NONE;
	file_size = 0;
	has_thumb = false;

	html_template.clear();
	sort_index = 0;
	meta.Clear();
	ameta.Clear();

	SetDateToNow();
}


/*
 * we're using the HtmlFilter only for those contents
 *
 */
bool Item::CanContentBeHtmlFiltered(Item::ContentType ct)
{
	return ct == ct_text || ct == ct_formatted_text || ct == ct_html || ct == ct_bbcode;
}

bool Item::CanContentBeHtmlFiltered()
{
	return CanContentBeHtmlFiltered(content_type);
}


} // namespace Winix

