/*
 * This file is a part of Winix
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/* 
 * Copyright (c) 2008-2018, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
 
#include "download.h"



namespace Winix
{


namespace Fun
{


Download::Download()
{
	fun.url = L"download";
	need_session = false;
}


void Download::MakeGet()
{
	// !! moze wywalic to no_item i wszedzie w takich miejscach dac poprostu permission_denied?
	if( !cur->request->is_item )
	{
		log << log1 << "Content: download function requires an item" << logend;
		cur->request->status = WINIX_ERR_NO_ITEM;
		return;
	}

	
	if( !system->HasReadAccess(cur->request->item)				||
		cur->request->item.file_type == WINIX_ITEM_FILETYPE_NONE ||
		cur->request->item.file_path.empty()						)
	{
		cur->request->status = WINIX_ERR_PERMISSION_DENIED;
		return;
	}

	cur->request->send_as_attachment = cur->request->IsParam(L"attachment");

	if( cur->request->item.has_thumb && cur->request->IsParam(L"thumb") )
		system->MakeFilePath(cur->request->item, cur->request->x_sendfile, true);
	else
		system->MakeFilePath(cur->request->item, cur->request->x_sendfile);
}




} // namespace

} // namespace Winix

