/*
 * This file is a part of Winix
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/* 
 * Copyright (c) 2010-2018, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
 
#include "functionbase.h"
#include "functions.h"


namespace Winix
{


FunctionBase::FunctionBase()
{
	follow_symlinks = true;
	template_index  = size_t(-1);
	need_ssl		= false;
	need_session	= true;

	fun.user_id    = -1;
	fun.group_id   = -1;
	fun.privileges = 07555;
	fun.parent_id  = -1;
	fun.id         = -1;
	fun.type       = Item::file;

	db = nullptr;
	functions = nullptr;
	templates = nullptr;
}


FunctionBase::~FunctionBase()
{
}



//void FunctionBase::SetConfig(Config * pconfig)
//{
//	config = pconfig;
//}


//void FunctionBase::SetCur(Cur * pcur)
//{
//	cur = pcur;
//}



void FunctionBase::SetDb(Db * pdb)
{
	db = pdb;
}


//void FunctionBase::SetSystem(System * psystem)
//{
//	system = psystem;
//}


void FunctionBase::SetFunctions(Functions * pfunctions)
{
	functions = pfunctions;
}


void FunctionBase::SetTemplates(Templates * ptemplates)
{
	templates = ptemplates;
}


//void FunctionBase::SetSynchro(Synchro * psynchro)
//{
//	synchro = psynchro;
//}


//void FunctionBase::SetSessionManager(SessionManager * pmanager)
//{
//	session_manager = pmanager;
//}


void FunctionBase::Init()
{
	// this method is called only once at the beginning
	// when winix starts
}


bool FunctionBase::HasAccess()
{
	// true by default
	return true;
}



void FunctionBase::MakePost()
{
	// do nothing by default
}


void FunctionBase::MakeGet()
{
	// do nothing by default
}

void FunctionBase::MakeDelete()
{
	// do nothing by default
}



} // namespace Winix


