/*
 * This file is a part of Winix
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/* 
 * Copyright (c) 2010-2014, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#include <algorithm>
#include "showthreads.h"
#include "functions.h"

namespace Winix
{



namespace Thread
{

ShowThreads::ShowThreads()
{
	fun.url = L"showthreads";
}


void ShowThreads::SetTDb(TDb * ptdb)
{
	tdb = ptdb;
}



void ShowThreads::SetThreadInfo(ThreadInfo * pthread_info)
{
	thread_info = pthread_info;
}




bool ShowThreads::HasAccess()
{
	if( cur->request->is_item )
		return false;

	if( system->mounts.pmount->type != thread_info->mount_type_thread )
		return false;

return true;
}




bool ShowThreads::Sort::operator()(const Item * item1, const Item * item2)
{
	if( sort_type == 0 )
	{
		// sorting by url
		return item1->url < item2->url;
	}
	else
	{
		// sorting by date
		return item1->date_creation > item2->date_creation;
	}
}




void ShowThreads::ReadFiles()
{
	// reading files
	DbItemQuery iq;
	iq.SetAll(false, false);
	iq.sel_url        = true;
	iq.sel_subject    = true;
	iq.sel_date       = true;
	iq.sel_user_id    = true;
	iq.sel_group_id   = true;
	iq.sel_guest_name = true;
	iq.sel_privileges = true;
	iq.sel_date       = true;
	iq.WhereParentId(cur->request->dir_tab.back()->id);
	iq.WhereType(Item::file);
	iq.WhereFileType(WINIX_ITEM_FILETYPE_NONE);

	db->GetItems(thread_info->item_tab, iq);
	system->CheckAccessToItems(thread_info->item_tab);
}



void ShowThreads::CreatePointers()
{
	// creating a pointers table
	thread_info->item_sort_tab.resize(thread_info->item_tab.size());

	for(size_t i=0 ; i<thread_info->item_tab.size() ; ++i)
		thread_info->item_sort_tab[i] = &thread_info->item_tab[i];
}


void ShowThreads::SortPointers()
{
	int sort_type = 1;

	if( cur->request->ParamValue(L"sort") == L"url" )
		sort_type = 0;

	std::vector<Item*> & table = thread_info->item_sort_tab;
	std::sort(table.begin(), table.end(), Sort(this, sort_type));
}



void ShowThreads::ReadThreads()
{
	// reading threads for the files
	file_id_tab.resize(thread_info->item_sort_tab.size());

	for(size_t i=0 ; i<thread_info->item_sort_tab.size() ; ++i)
		file_id_tab[i] = thread_info->item_sort_tab[i]->id;

	tdb->GetThreads(file_id_tab, thread_info->thread_tab);
}



void ShowThreads::MakeGet()
{
	thread_info->Clear();

	ReadFiles();
	CreatePointers();
	SortPointers();
	ReadThreads();
}





} // namespace

} // namespace Winix

